
#include "comunications.h"
#include "serialCom.h"


serialCom::serialCom() {
	comunications::getRef()->appendInterface(this, "RS232");
	r_connectionIsEnabled=false;
	bufferIndex=0;
	preCnt=0;
	epiCnt=0;
}

serialCom::~serialCom() {

}

comStatus serialCom::initialize() {

	return connectionOk;
}

int serialCom::sendBuffer(char* buffer, int length) {
	int nBytes=0;
	DWORD n = 0;

	if (WriteFile(r_settings->descriptor, buffer, length, &n, NULL)==0){
		nBytes=-1;
	}

	nBytes = n;

	return nBytes;
}

int serialCom::receiveBuffer(char* rxBuffer) {

	if (!r_connectionIsEnabled){
		return 0;
	}

	int nBytes = 0;

	DWORD x;
	COMSTAT cs;

	// Actualizar COMSTAT, sirve para averiguar el numero de bytes en el buffer de entrada
	ClearCommError(r_settings->descriptor, &x, &cs);
	nBytes = cs.cbInQue;

	if (r_settings->controlType == _SOFTWARE_CONTROL_){

		ReadFile(r_settings->descriptor, auxBuffer, nBytes, &x, NULL);
		for (unsigned int i=0;i<nBytes;i++){
			incommingBuffer.push(auxBuffer[i]);
		}

		int nFifoElements = incommingBuffer.getNumberOfElements();
		unsigned int len=0;
		char c;

		for (unsigned int i=0; i<nFifoElements && !len; i++){
			incommingBuffer.pop(&c);
			len = getFrame(c);
			if (len){
				memcpy(rxBuffer,frameBuffer,len);
			}
		}

		nBytes=len;
	}
	else{
		//printf("\nSE RECIBEN %d BYTES:\n", cs.cbInQue);
		ReadFile(r_settings->descriptor, rxBuffer, nBytes, &x, NULL);
	}

	//Si no hay datos que procesar, no rasques indiscriminadamente los recursos del procesador.
	if (nBytes==0){
		Sleep(100);
	}

	return nBytes;
}

void serialCom::configure(void* configuration){

	r_settings = (stSerialPortConfiguration*)configuration;

	printf("EPilogue size: %d\n",r_settings->epilogueSize);
	printf("Preamble size: %d\n",r_settings->preambleSize);

	r_connectionIsEnabled =true;
	printf("RS232 CONFIG\n");
	fflush(stdout);

}

int serialCom::getFrame(char c) {

	static unsigned int preCnt=0;
	static unsigned int epiCnt=0;
//	static int bufferIndex=0;
	static bool gettingData = false;

	if (r_settings->preambleSize==0){
		gettingData=true;
	}

	if (gettingData == false) {
		//Buscamos preambulo.
		if (c == r_settings->preamble[preCnt]) {
			preCnt++;
		} else {
			preCnt = 0;
		}

		if (preCnt == r_settings->preambleSize) {
			//cout << "Preambulo detectado!" << endl << flush;
			preCnt = 0;
			bufferIndex=0;
			gettingData = true;
		}
	} else {
		frameBuffer[bufferIndex] = c;

		if (bufferIndex<_RS232_FRAME_LENGTH_){
			bufferIndex++;
		}
		else{
			//El tamao de la trama excede el tamao permitido... RESETEAMOS.
			printf("RESETEAMOS FRAMEBUFFER!\n");
			fflush(stdout);

			bufferIndex=0;
		}

		//buscamos epilogo.
		if (c == r_settings->epilogue[epiCnt]) {
			epiCnt++;
		} else {
			epiCnt = 0;
		}

		if (epiCnt == r_settings->epilogueSize) {
			//out << "Epilogo detectado!" << endl << flush;
			epiCnt = 0;
			gettingData = false;

			//Quitamos del buffer de datos el epilogo detectado.
			bufferIndex -= r_settings->epilogueSize;
			if (bufferIndex<0){
				bufferIndex=0;
			}

			int retorno=bufferIndex;
			bufferIndex=0; //Reseteamos la trama para la proxima deteccion.
			return retorno;
		}
	}

	return 0;
}

